"""
This program is free software: you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""


import logging
import pandas as pd
from get_biomes.utils import (
    get_arguments,
    concat_df,
    get_mgnify_data,
    get_ena_data,
    create_output_files,
    search_str,
)
from get_biomes.defaults import API_BASE
from jsonapi_client import Filter, Session
from urllib.parse import urlencode
import urllib.parse
import tqdm
from multiprocessing import Pool
import os
from pathlib import Path
from functools import partial

log = logging.getLogger("my_logger")


def search(args):

    logging.basicConfig(
        level=logging.DEBUG,
        format="%(levelname)s ::: %(asctime)s ::: %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    args = get_arguments()

    # Check if outfile exists and remove it
    # get full path to outfile
    logging.getLogger("my_logger").setLevel(
        logging.DEBUG if args.debug else logging.INFO
    )
    logging.getLogger("urllib3").setLevel(logging.ERROR)
    logging.getLogger("jsonapi_client").setLevel(logging.ERROR)

    biomes = pd.read_csv(args.biomes, sep="\t", header=None)
    biomes = biomes[0].tolist()
    output_files = create_output_files(args.prefix, args.biomes, biomes)

    if args.clean_output:
        logging.info("Removing old output files...")
        for file in output_files:
            if os.path.exists(output_files[file]):
                os.remove(output_files[file])
    # API call
    with Session(API_BASE) as session:

        # configure the filters
        if args.mg_filter is not None:
            params = args.mg_filter
            api_filter = Filter(urlencode(params))

        for biome in biomes:
            log.info(f"Processing biome: {biome}...")
            if Path(output_files[biome]).is_file():
                log.info(f"File {output_files[biome]} already exists, skipping")
                continue
            log.info("::: Gathering sample info from MGnify...")
            query = urllib.parse.quote(f"biomes/{biome}/samples")
            if args.mg_filter is not None:
                samples = [
                    get_mgnify_data(x)
                    for x in tqdm.tqdm(
                        session.iterate(query, api_filter),
                        desc="Samples processed",
                        leave=False,
                    )
                ]
            else:
                samples = [
                    get_mgnify_data(x)
                    for x in tqdm.tqdm(
                        session.iterate(query),
                        desc="Samples processed",
                        leave=False,
                    )
                ]

            samples = concat_df(samples)

            log.info("::: Getting ENA links...")

            p = Pool(args.threads)
            dfs = list(
                tqdm.tqdm(
                    p.imap_unordered(
                        partial(get_ena_data, filter_conditions=args.ena_filter),
                        samples.accession,
                    ),
                    total=len(samples.accession),
                    leave=False,
                    ncols=80,
                    desc="Samples processed",
                )
            )
            p.close()
            p.join()
            # print(dfs)
            dfs = [x for x in dfs if x is not None]
            if len(dfs) > 0:
                dfs = concat_df(dfs)
                dfs = samples.merge(dfs, on="sample_accession")
            else:
                columns = [
                    "study_accession",
                    "experiment_accession",
                    "run_accession",
                    "read_count",
                    "instrument_model",
                    "instrument_platform",
                    "library_layout",
                    "library_strategy",
                    "library_selection",
                    "fastq_ftp",
                ]
                dfs[columns] = None

            dfs["query_biome"] = biome

            if args.exclude_terms is not None:
                if len(args.exclude_terms) > 1:
                    terms = "|".join(args.exclude_terms)
                else:
                    terms = args.exclude_terms[0]

                dfs = search_str(terms, dfs, invert=True)

            logging.info("::: Writing output file...")
            dfs.to_csv(
                output_files[biome],
                sep="\t",
                index=False,
            )

    if args.combine:
        logging.info("Combining output files...")
        if os.path.exists(output_files["combined"]):
            os.remove(output_files["combined"])
        dfs = []
        for file in output_files.values():
            if file == output_files["combined"]:
                continue
            dfs.append(pd.read_csv(file, sep="\t"))
        dfs = concat_df(dfs)
        dfs.to_csv(output_files["combined"], sep="\t", index=False)

    logging.info("ALL DONE.")



from os import makedirs
from os.path import basename
from os.path import join
from get_biomes.utils import get_arguments, fast_flatten
import logging
import pandas as pd
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import requests
import tqdm
import os
from functools import partial
from multiprocessing.pool import ThreadPool


def download_url(url, path):
    retry_strategy = Retry(
        total=5,
        backoff_factor=1,
        status_forcelist=[429, 500, 502, 503, 504, 403],
        allowed_methods=["HEAD", "GET", "OPTIONS"],
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    http = requests.Session()
    http.mount("https://", adapter)
    http.mount("http://", adapter)
    try:
        file_name = os.path.join(path, basename(url))
        with http.get(f"http://{url}", timeout=10, stream=True) as r:
            if r.status_code == requests.codes.ok:
                r.raise_for_status()
                with open(file_name, "wb") as f:
                    for chunk in r.iter_content(chunk_size=8192):
                        f.write(chunk)
                return basename(url), True, r.status_code, url
            else:
                return basename(url), False, r.status_code, url
    except Exception as e:
        return basename(url), False, e, url


log = logging.getLogger("my_logger")


def download(args):

    logging.basicConfig(
        level=logging.DEBUG,
        format="%(levelname)s ::: %(asctime)s ::: %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    args = get_arguments()

    # Check if outfile exists and remove it
    # get full path to outfile
    logging.getLogger("my_logger").setLevel(
        logging.DEBUG if args.debug else logging.INFO
    )
    logging.getLogger("urllib3").setLevel(logging.ERROR)
    logging.getLogger("jsonapi_client").setLevel(logging.ERROR)

    biomes = pd.read_csv(args.input, sep="\t", header=0)
    urls = fast_flatten(biomes["fastq_ftp"].str.split(";", expand=True).values.tolist())

    # if output directory does not exist, create it if not delete
    if not os.path.exists(args.outdir):
        makedirs(args.outdir)

    # Check if download_report file exists
    download_report = os.path.join(args.outdir, "download_report.tsv")
    if os.path.exists(download_report):
        report = pd.read_csv(download_report, sep="\t", header=0)
        report_success = report[report["success"] == True]
        report_failed = report[report["success"] == False]
        log.info(
            f"Found {report_success.shape[0]} successful downloads and {report_failed.shape[0]} failed downloads"
        )
        urls = list(set(urls) - set(report_success["url"]))
        if len(urls) == 0:
            logging.info("All files have been downloaded successfully")
            return

    if args.clean_output and os.path.exists(args.outdir):
        log.info("Output directory already exists, deleting it...")
        for file in os.listdir(args.outdir):
            os.remove(os.path.join(args.outdir, file))

    files = []
    log.info("Downloading files...")
    # Run 5 multiple threads. Each call will take the next element in urls list
    results = ThreadPool(args.threads).imap_unordered(
        partial(download_url, path=args.outdir), urls
    )

    for r in tqdm.tqdm(
        results, total=len(urls), desc="Files downloaded", ncols=80, leave=False
    ):
        files.append(r)

    if os.path.exists(download_report):
        report = pd.DataFrame(files, columns=["filename", "success", "error", "url"])
        if report_success.shape[0] > 0:
            report = pd.concat([report, report_success], axis=0)
    else:
        report = pd.DataFrame(files, columns=["filename", "success", "error", "url"])

    report.to_csv(download_report, sep="\t", index=False)
    log.info("Done!")



import argparse
import sys
import gzip
import os
import logging
import pandas as pd
from multiprocessing import Pool
from functools import partial
from os import devnull
import tqdm
from get_biomes import __version__
import time
from itertools import chain
import io
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import requests
from contextlib import contextmanager, redirect_stderr, redirect_stdout
import json
from pathlib import Path
import re
from get_biomes.defaults import (
    mgnify_filters,
    ena_filters,
    API_BASE_ENA,
)

log = logging.getLogger("my_logger")
log.setLevel(logging.INFO)
timestr = time.strftime("%Y%m%d-%H%M%S")


def is_debug():
    return logging.getLogger("my_logger").getEffectiveLevel() == logging.DEBUG


class SplitArgs(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        try:
            values = list(map(str, values.split(",")))
        except ValueError:
            raise argparse.ArgumentError(self, "invalid values: %r" % values)
        # check all elements in list are greater than 0
        if not all(str(x) for x in values):
            raise argparse.ArgumentError(self, "all elements must be a string")
        setattr(namespace, self.dest, values)


# From https://stackoverflow.com/a/63684031
def search_str(regex: str, df, case=False, invert=False):
    """Search all the text columns of `df`, return rows with any matches."""
    textlikes = df.select_dtypes(include=[object, "string"])
    if invert:
        return df[
            ~textlikes.apply(
                lambda column: column.str.contains(
                    regex, regex=True, case=case, na=False
                )
            ).any(axis=1)
        ]
    else:
        return df[
            textlikes.apply(
                lambda column: column.str.contains(
                    regex, regex=True, case=case, na=False
                )
            ).any(axis=1)
        ]


def fast_flatten(input_list):
    return list(chain.from_iterable(input_list))


def concat_df(frames):
    COLUMN_NAMES = frames[0].columns
    df_dict = dict.fromkeys(COLUMN_NAMES, [])
    for col in COLUMN_NAMES:
        extracted = (frame[col] for frame in frames)
        # Flatten and save to df_dict
        df_dict[col] = fast_flatten(extracted)
    df = pd.DataFrame.from_dict(df_dict)[COLUMN_NAMES]
    return df


def check_values(val, minval, maxval, parser, var):
    value = float(val)
    if value < minval or value > maxval:
        parser.error(
            "argument %s: Invalid value %s. Range has to be between %s and %s!"
            % (
                var,
                value,
                minval,
                maxval,
            )
        )
    return value


# From: https://note.nkmk.me/en/python-check-int-float/
def is_integer(n):
    try:
        float(n)
    except ValueError:
        return False
    else:
        return float(n).is_integer()


# function to check if the input value has K, M or G suffix in it
def check_suffix(val, parser, var):
    units = ["K", "M", "G"]
    unit = val[-1]
    value = int(val[:-1])

    if is_integer(value) & (unit in units) & (value > 0):
        return val
    else:
        parser.error(
            "argument %s: Invalid value %s. Memory has to be an integer larger than 0 with the following suffix K, M or G"
            % (var, val)
        )


def get_compression_type(filename):
    """
    Attempts to guess the compression (if any) on a file using the first few bytes.
    http://stackoverflow.com/questions/13044562
    """
    magic_dict = {
        "gz": (b"\x1f", b"\x8b", b"\x08"),
        "bz2": (b"\x42", b"\x5a", b"\x68"),
        "zip": (b"\x50", b"\x4b", b"\x03", b"\x04"),
    }
    max_len = max(len(x) for x in magic_dict)

    unknown_file = open(filename, "rb")
    file_start = unknown_file.read(max_len)
    unknown_file.close()
    compression_type = "plain"
    for file_type, magic_bytes in magic_dict.items():
        if file_start.startswith(magic_bytes):
            compression_type = file_type
    if compression_type == "bz2":
        sys.exit("Error: cannot use bzip2 format - use gzip instead")
        sys.exit("Error: cannot use zip format - use gzip instead")
    return compression_type


def get_open_func(filename):
    if get_compression_type(filename) == "gz":
        return gzip.open
    else:  # plain text
        return open


# From: https://stackoverflow.com/a/11541450
def is_valid_file(parser, arg, var):
    if not os.path.exists(arg):
        parser.error("argument %s: The file %s does not exist!" % (var, arg))
    else:
        return arg


# From https://stackoverflow.com/a/59617044/15704171
def convert_list_to_str(lst):
    n = len(lst)
    if not n:
        return ""
    if n == 1:
        return lst[0]
    return ", ".join(lst[:-1]) + f" or {lst[-1]}"


def is_valid_filter(parser, arg, var, filter_list):
    arg = json.loads(arg)
    # check if the dictionary keys are in the mdmg header list
    if var == "--ena-filter":
        for key in arg.keys():
            if key not in filter_list.keys():
                trms = convert_list_to_str(list(filter_list.keys()))
                parser.error(
                    f"argument {var}: Invalid key {key}. Valid keys are {trms}!"
                )
        for key in arg.keys():
            if key == "read_count":
                if int(arg[key]) < 1:
                    parser.error(
                        f"argument {var}: Invalid value for {arg[key]}. Valid values have to be larger than 1!"
                    )
            else:
                if arg[key] not in ena_filters[key]:
                    trms = convert_list_to_str(list(ena_filters[key]))
                    parser.error(
                        f"argument {var}: Invalid value for {key}. Valid values are {trms}!"
                    )
    else:
        for key in arg.keys():
            if key not in filter_list:
                parser.error(
                    f"argument {var}: Invalid value {key}.\n"
                    f"Valid values are: {convert_list_to_str(filter_list)}"
                )
    return arg


defaults = {
    "threads": 1,
    "outfile": "output.tsv",
    "prefix": None,
    "min_read_count": 0,
    "mg_filter": {
        "experiment_type": "metagenomic",
        "instrument_platform": "illumina",
    },
    "ena_filter": {
        "library_layout": "PAIRED",
        "library_strategy": "WGS",
        "library_source": "METAGENOMIC",
        "library_selection": "RANDOM",
        "read_count": 10000000,
    },
}

help_msg = {
    "biomes": "A txt file containing MGnify biomes. Ex: root:Environmental:Aquatic:Marine",
    "threads": "Number of threads to use",
    "mg_filter": f"Key-value pairs to filter the MGnify metadata. Valid values are: {convert_list_to_str(mgnify_filters)}",
    "ena_filter": f"Key-value pairs to filter the ENA metadata. Valid values are: {convert_list_to_str(list(ena_filters.keys()))}",
    "prefix": "Prefix for the output file",
    "outfile": "Output file name",
    "paired": "Only keep paired-end samples",
    "combine": "Combine all output files into one",
    "exclude_terms": "A comma-separated list of terms to exclude from the metadata",
    "clean": "Remove existing output files",
    "input": "A txt file containing the biome' fastq files to download",
    "outdir": "The directory to save the fastq files",
    "help": "Help message",
    "debug": "Print debug messages",
    "version": "Print program version",
}


def get_arguments(argv=None):
    parser = argparse.ArgumentParser(
        description="A simple tool to get biomes related data from MGnify and ENA",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        add_help=False,
    )
    parser.add_argument(
        "-h",
        "--help",
        action="help",
        default=argparse.SUPPRESS,
        help="show this help message and exit",
    )
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s " + __version__,
        help=help_msg["version"],
    )

    # Same subparsers as usual
    sub_parsers = parser.add_subparsers(
        help="positional arguments",
        dest="action",
    )

    # Create parent subparser. Note `add_help=False` and creation via `argparse.`
    parent_parser = argparse.ArgumentParser(add_help=False)
    optional = parent_parser._action_groups.pop()
    optional.add_argument(
        "--debug", dest="debug", action="store_true", help=help_msg["debug"]
    )

    # create the parser sub-commands
    parser_search = sub_parsers.add_parser(
        "search",
        help="Search for biomes in MGnify and ENA",
        parents=[parent_parser],
    )
    parser_download = sub_parsers.add_parser(
        "download",
        help="Download the biomes gathered by the subcommand search",
        parents=[parent_parser],
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    search_required = parser_search.add_argument_group("search required arguments")
    search_optional = parser_search.add_argument_group("search optional arguments")
    search_required.add_argument(
        "-b",
        "--biomes",
        dest="biomes",
        type=lambda x: is_valid_file(parser, x, "--biomes"),
        help=help_msg["biomes"],
        required=True,
    )
    search_optional.add_argument(
        "--mgnify-filter",
        type=lambda x: is_valid_filter(parser, x, "--mgnify-filter", mgnify_filters),
        dest="mg_filter",
        default=None,
        help=help_msg["mg_filter"],
        required=False,
    )
    search_optional.add_argument(
        "--ena-filter",
        type=lambda x: is_valid_filter(parser, x, "--ena-filter", ena_filters),
        dest="ena_filter",
        default=None,
        help=help_msg["ena_filter"],
        required=False,
    )
    search_optional.add_argument(
        "--exclude-terms",
        default=None,
        dest="exclude_terms",
        action=SplitArgs,
        help=help_msg["exclude_terms"],
    )
    search_optional.add_argument(
        "-p",
        "--prefix",
        type=str,
        default=defaults["prefix"],
        dest="prefix",
        help=help_msg["prefix"],
    )
    search_optional.add_argument(
        "-t",
        "--threads",
        type=lambda x: int(
            check_values(x, minval=1, maxval=1000, parser=parser, var="--threads")
        ),
        dest="threads",
        default=1,
        help=help_msg["threads"],
        required=False,
    )
    search_optional.add_argument(
        "--combine",
        dest="combine",
        action="store_true",
        help=help_msg["combine"],
        required=False,
    )
    search_optional.add_argument(
        "--clean",
        dest="clean_output",
        action="store_true",
        help=help_msg["clean"],
        required=False,
    )

    download_required = parser_download.add_argument_group(
        "download required arguments"
    )
    download_optional = parser_download.add_argument_group(
        "download optional arguments"
    )
    download_required.add_argument(
        "-i",
        "--input",
        dest="input",
        type=lambda x: is_valid_file(parser, x, "--input"),
        help=help_msg["biomes"],
        required=True,
    )
    download_optional.add_argument(
        "-o",
        "--outdir",
        type=str,
        dest="outdir",
        help=help_msg["outdir"],
    )
    download_optional.add_argument(
        "--clean",
        dest="clean_output",
        action="store_true",
        help=help_msg["clean"],
        required=False,
    )
    download_optional.add_argument(
        "-t",
        "--threads",
        type=lambda x: int(
            check_values(x, minval=1, maxval=1000, parser=parser, var="--threads")
        ),
        dest="threads",
        default=1,
        help=help_msg["threads"],
        required=False,
    )

    args = parser.parse_args(None if sys.argv[1:] else ["-h"])
    return args


@contextmanager
def suppress_stdout():
    """A context manager that redirects stdout and stderr to devnull"""
    with open(devnull, "w") as fnull:
        with redirect_stderr(fnull) as err, redirect_stdout(fnull) as out:
            yield (err, out)


def applyParallel(dfGrouped, func, threads, parms):
    p = Pool(threads)
    func = partial(func, parms=parms)
    ret_list = tqdm.tqdm(
        p.map(func, [group for name, group in dfGrouped]),
        total=len([group for name, group in dfGrouped]),
    )
    p.close()
    p.join()
    return pd.concat(ret_list)


def initializer(init_data):
    global parms
    parms = init_data


def do_parallel(parms, lst, func, threads):
    if is_debug():
        dfs = list(map(partial(func, parms=parms), lst))
    else:
        p = Pool(threads, initializer=initializer, initargs=(parms,))
        c_size = calc_chunksize(threads, len(lst))
        dfs = list(
            tqdm.tqdm(
                p.imap_unordered(
                    partial(func, parms=parms),
                    lst,
                    chunksize=c_size,
                ),
                total=len(lst),
                leave=False,
                ncols=80,
                desc=f"Components processed",
            )
        )
        p.close()
        p.join()
    return concat_df(dfs)


# from https://stackoverflow.com/questions/53751050/python-multiprocessing-understanding-logic-behind-chunksize/54032744#54032744
def calc_chunksize(n_workers, len_iterable, factor=4):
    """Calculate chunksize argument for Pool-methods.

    Resembles source-code within `multiprocessing.pool.Pool._map_async`.
    """
    chunksize, extra = divmod(len_iterable, n_workers * factor)
    if extra:
        chunksize += 1
    return chunksize


def get_mgnify_data(sample):
    # instrument_model = None
    # sequencing_method = None
    # investigation_type = None
    # for k in sample.sample_metadata:
    #     if k["key"] == "instrument model":
    #         instrument_model = k["value"]
    #     elif k["key"] == "sequencing method":
    #         sequencing_method = k["value"]
    #     elif k["key"] == "investigation type":
    #         investigation_type = k["value"]
    d = {
        "accession": sample.accession,
        "sample_accession": sample.biosample,
        "sample_name": sample.sample_name,
        "longitude": sample.longitude,
        "latitude": sample.latitude,
        "geo_loc_name": sample.geo_loc_name,
        "studies": ",".join([study.accession for study in sample.studies]),
        "biome": sample.biome.id,
        "sample_desc": sample.sample_desc,
        "environment_biome": sample.environment_biome,
        "environment_feature": sample.environment_feature,
        "environment_material": sample.environment_material,
    }

    df_mg = pd.DataFrame({k: [v] for k, v in d.items()})
    return df_mg


def get_data(url):
    # Get the data from the API
    retry_strategy = Retry(
        total=5,
        backoff_factor=1,
        status_forcelist=[429, 500, 502, 503, 504, 403],
        allowed_methods=["HEAD", "GET", "OPTIONS"],
    )
    adapter = HTTPAdapter(max_retries=retry_strategy)
    http = requests.Session()
    http.mount("https://", adapter)
    http.mount("http://", adapter)
    r = http.get(url, timeout=10)
    # Parse the data
    if r.status_code != 200:
        r = io.StringIO("")
    else:
        r = io.StringIO(r.content.decode("utf-8"))
    return r


def get_ena_data(accession, filter_conditions):
    ena_url = f"{API_BASE_ENA}filereport?accession={accession}&result=read_run&fields=sample_accession,study_accession,experiment_accession,run_accession,read_count,instrument_model,instrument_platform,library_layout,library_strategy,library_selection,library_source,fastq_ftp&format=tsv&download=true&limit=0"
    data = get_data(ena_url)
    data.seek(0)
    try:
        df = pd.read_csv(data, sep="\t")
        if filter_conditions is not None:
            min_read_count = 0
            if filter_conditions["read_count"]:
                min_read_count = int(filter_conditions["read_count"])
                del filter_conditions["read_count"]
            df = df.loc[
                (df[list(filter_conditions)] == pd.Series(filter_conditions)).all(
                    axis=1
                )
            ]
            if min_read_count > 0:
                df = df[df["read_count"] >= min_read_count]
    except pd.errors.EmptyDataError:
        df = None
    return df
    # return pd.read_csv(ena_url, sep="\t")


def create_output_files(prefix, input, biomes):
    if prefix is None:
        prefix = Path(input).resolve().stem.split(".")[0]

    out_files = {}
    out_files["combined"] = f"{prefix}__combined.tsv"
    for biome in biomes:
        # remove non alphanumeric characters from biome name
        biome_enc = re.sub(r"\W+", "_", biome)
        out_files[biome] = f"{prefix}__{biome_enc}.tsv"

    return out_files



"""
This program is free software: you can redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""


import logging


from get_biomes.search import search
from get_biomes.download import download
from get_biomes.utils import (
    get_arguments,
)

log = logging.getLogger("my_logger")


def main():

    logging.basicConfig(
        level=logging.DEBUG,
        format="%(levelname)s ::: %(asctime)s ::: %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    args = get_arguments()

    # Check if outfile exists and remove it
    # get full path to outfile
    logging.getLogger("my_logger").setLevel(
        logging.DEBUG if args.debug else logging.INFO
    )
    logging.getLogger("urllib3").setLevel(logging.ERROR)
    logging.getLogger("jsonapi_client").setLevel(logging.ERROR)

    if args.action == "search":
        search(args)
    elif args.action == "download":
        download(args)


if __name__ == "__main__":
    main()



API_BASE = "https://www.ebi.ac.uk/metagenomics/api/v1"
API_BASE_ENA = "https://www.ebi.ac.uk/ena/portal/api/"

mgnify_filters = [
    "experiment_type",
    "biome_name",
    "lineage",
    "geo_loc_name",
    "latitude_gte",
    "latitude_lte",
    "longitude_gte",
    "longitude_lte",
    "species",
    "instrument_model",
    "instrument_platform",
    "metadata_key",
    "metadata_value_gte",
    "metadata_value_lte",
    "metadata_value",
    "environment_material",
    "environment_feature",
    "study_accession",
    "include",
]
ena_library_strategy = [
    "WGS",
    "WGA",
    "WXS",
    "RNA-Seq",
    "ssRNA-seq",
    "miRNA-Seq",
    "ncRNA-Seq",
    "FL-cDNA",
    "EST",
    "Hi-C",
    "ATAC-seq",
    "WCS",
    "RAD-Seq",
    "CLONE",
    "POOLCLONE",
    "AMPLICON",
    "CLONEEND",
    "FINISHING",
    "ChIP-Seq",
    "MNase-Seq",
    "DNase-Hypersensitivity",
    "Bisulfite-Seq",
    "CTS",
    "MRE-Seq",
    "MeDIP-Seq",
    "MBD-Seq",
    "Tn-Seq",
    "VALIDATION",
    "FAIRE-seq",
    "SELEX",
    "RIP-Seq",
    "ChIA-PET",
    "Synthetic-Long-Read",
    "Targeted-Capture",
    "Tethered Chromatin Conformation Capture",
    "OTHER",
    "NOMe-Seq",
    "ChM-Seq",
    "GBS",
]

ena_library_source = [
    "GENOMIC",
    "GENOMIC SINGLE CELL",
    "TRANSCRIPTOMIC",
    "TRANSCRIPTOMIC SINGLE CELL",
    "METAGENOMIC",
    "METATRANSCRIPTOMIC",
    "SYNTHETIC",
    "VIRAL RNA",
    "OTHER",
]

ena_library_selection = [
    "RANDOM",
    "PCR",
    "RANDOM PCR",
    "RT-PCR",
    "HMPR",
    "MF",
    "repeat fractionation",
    "size fractionation",
    "MSLL",
    "cDNA",
    "cDNA_randomPriming",
    "cDNA_oligo_dT",
    "PolyA",
    "Oligo-dT",
    "Inverse rRNA",
    "Inverse rRNA selection",
    "ChIP",
    "ChIP-Seq",
    "MNase",
    "DNase",
    "Hybrid Selection",
    "Reduced Representation",
    "Restriction Digest",
    "5-methylcytidine antibody",
    "MBD2 protein methyl-CpG binding domain",
    "CAGE",
    "RACE",
    "MDA",
    "padlock probes capture method",
    "other",
    "unspecified",
]

ena_library_layout = ["PAIRED", "SINGLE"]

ena_instrument_platform = [
    "LS454",
    "ILLUMINA",
    "HELICOS",
    "ABI_SOLID",
    "COMPLETE_GENOMICS",
    "BGISEQ",
    "OXFORD_NANOPORE",
    "PACBIO_SMRT",
    "ION_TORRENT",
    "CAPILLARY",
    "DNBSEQ",
]

ena_instrument_model = [
    "454 GS",
    "454 GS 20",
    "454 GS FLX",
    "454 GS FLX Titanium",
    "454 GS FLX+",
    "454 GS Junior",
    "AB 310 Genetic Analyzer",
    "AB 3130 Genetic Analyzer",
    "AB 3130xL Genetic Analyzer",
    "AB 3500 Genetic Analyzer",
    "AB 3500xL Genetic Analyzer",
    "AB 3730 Genetic Analyzer",
    "AB 3730xL Genetic Analyzer",
    "AB 5500 Genetic Analyzer",
    "AB 5500xl Genetic Analyzer",
    "AB 5500xl-W Genetic Analysis System",
    "AB SOLiD 3 Plus System",
    "AB SOLiD 4 System",
    "AB SOLiD 4hq System",
    "AB SOLiD PI System",
    "AB SOLiD System",
    "AB SOLiD System 2.0",
    "AB SOLiD System 3.0",
    "BGISEQ-500",
    "Complete Genomics",
    "DNBSEQ-G400",
    "DNBSEQ-G400 FAST",
    "DNBSEQ-G50",
    "DNBSEQ-T7",
    "GridION",
    "Helicos HeliScope",
    "HiSeq X Five",
    "HiSeq X Ten",
    "Illumina Genome Analyzer",
    "Illumina Genome Analyzer II",
    "Illumina Genome Analyzer IIx",
    "Illumina HiScanSQ",
    "Illumina HiSeq 1000",
    "Illumina HiSeq 1500",
    "Illumina HiSeq 2000",
    "Illumina HiSeq 2500",
    "Illumina HiSeq 3000",
    "Illumina HiSeq 4000",
    "Illumina iSeq 100",
    "Illumina MiniSeq",
    "Illumina MiSeq",
    "Illumina NovaSeq 6000",
    "Ion Torrent PGM",
    "Ion Torrent Proton",
    "Ion Torrent S5",
    "Ion Torrent S5 XL",
    "MinION",
    "NextSeq 500",
    "NextSeq 550",
    "PacBio RS",
    "PacBio RS II",
    "PromethION",
    "Sequel",
    "Sequel II",
    "unspecified",
]

ena_filters = {
    "read_count": [],
    "instrument_model": ena_instrument_model,
    "instrument_platform": ena_instrument_platform,
    "library_layout": ena_library_layout,
    "library_strategy": ena_library_strategy,
    "library_selection": ena_library_selection,
    "library_source": ena_library_source,
}
